﻿local L = AceLibrary("AceLocale-2.2"):new("GridAutoFrameSize")
local GridL = AceLibrary("AceLocale-2.2"):new("Grid")
local BZ = LibStub("LibBabble-Zone-3.0"):GetReverseLookupTable()

local math_ceil = math.ceil
local math_max = math.max
local string_find = string.find

local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers
local GetRaidRosterInfo = GetRaidRosterInfo
local GetCurrentDungeonDifficulty = GetCurrentDungeonDifficulty
local UnitExists = UnitExists

GridAutoFrameSize = Grid:NewModule("GridAutoFrameSize")
local GridAutoFrameSize = GridAutoFrameSize
local GridLayout = Grid:GetModule("GridLayout")

GridAutoFrameSize.defaultDB = {
	autoSizeFrames = true,
	autoZoneSize = false,
	withPets = false,
	forcePets = false,
	lastLayout = nil,
}

local zoneLock = false

local battlegrounds = {
  ["Alterac Valley"] = 40,
  ["Arathi Basin"] = 15,
  ["Eye of the Storm"] = 15,
  ["Strand of the Ancients"] = 15,
  ["Warsong Gulch"] = 10,
}

local instances = {
  ["Naxxramas"] = true,
  ["The Eye of Eternity"] = true,
  ["The Obsidian Sanctum"] = true,
  ["Vault of Archavon"] = true,
}

-- Returns mode of instance, 10 for normal, 25 for heroic
local function getMode()
	if not IsInInstance() then
		return 0
	end

  local mode = GetCurrentDungeonDifficulty()
  if mode == 1 then
    return 10
  elseif mode == 2 then
    return 25
  else
    return 0
  end
end

-- Returns max raid size of given instance, locale independent
local function getPredefinedZoneSize(zone)
  local englishZone = BZ[zone]

  if battlegrounds[englishZone] then
    return battlegrounds[englishZone]
  end
  
  if instances[englishZone] then
    return getMode()
  end
  
  return 0
end

-- Returns localized name of given layout from Grid's localization data
local function getLocalizedLayout(layout)
	if layout == nil then
		return nil
	end
	
	if GridL[layout] == nil then
		return layout
	end
	
	return GridL[layout]
end

-- Depending on options, returns layout name with or without pets
local function getWithPetsLayout(layout)
	if GridAutoFrameSize.db.profile.forcePets then
		return layout.." w/Pets"
	elseif GridAutoFrameSize.db.profile.withPets and UnitExists("pet") then
		return layout.." w/Pets"
	else
		return layout
	end
end

function GridAutoFrameSize:OnEnable()
	local opt = Grid.options.args
	opt["gridAutoFrameSize"] = {
		type = "group",
		name = "GridAutoFrameSize",
		desc = "GridAutoFrameSize",
		args = {
			autoSizeFrames = {
				type = "toggle",
				name = L["AUTO_SIZE"],
				desc = L["AUTO_SIZE_DESC"],
				order = 1,
				get = function ()	return self.db.profile.autoSizeFrames	end,
				set = function ()
					self.db.profile.autoSizeFrames = not self.db.profile.autoSizeFrames
					self:CheckRoster()
				end,
			},
			autoZoneSize = {
				type = "toggle",
				name = L["ZONE_SIZE"],
				desc = L["ZONE_SIZE_DESC"],
				order = 2,
				get = function ()	return self.db.profile.autoZoneSize end,
				set = function ()
					self.db.profile.autoZoneSize = not self.db.profile.autoZoneSize
					self:CheckZone()
				end,
			},
			withPets = {
				type = "toggle",
				name = L["WITH_PETS"],
				desc = L["WITH_PETS_DESC"],
				order = 3,
				get = function() return self.db.profile.withPets end,
				set = function() 
					self.db.profile.withPets = not self.db.profile.withPets
					self:CheckRoster()
				end,
			},
			forcePets = {
				type = "toggle",
				name = L["FORCE_PETS"],
				desc = L["FORCE_PETS_DESC"],
				order = 4,
				get = function() return self.db.profile.forcePets end,
				set = function() 
					self.db.profile.forcePets = not self.db.profile.forcePets
					self:CheckRoster()
				end,
			},
		},
	}

	self:RegisterBucketEvent("Grid_RosterUpdated", 2, "CheckRoster")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "CheckZone")

	self:CheckZone()
end

function GridAutoFrameSize:CheckZone()
	local zone = GetRealZoneText()
  local size = getPredefinedZoneSize(zone)

	if size ~= 0 then
		local newLayout = "By Group "..size
		if size ~= 40 then
			newLayout = getWithPetsLayout(newLayout)
		end
		
		newLayout = getLocalizedLayout(newLayout)

		if not newLayout then	
			return
		end

		zoneLock = true
		self.db.profile.lastLayout = newLayout
		GridLayout:LoadLayout(newLayout)
	else
		zoneLock = false
	end
end

function GridAutoFrameSize:CheckRoster()
	-- dont change layout, when auto frame size is disabled
	if not self.db.profile.autoSizeFrames then
		return
	end
	
	local layoutSize = 0
	local newLayout = nil
	local raidSize = GetNumRaidMembers()
	if raidSize > 0 then
		
		local maxGroup = math_ceil(raidSize / 5)
		for i = 1, raidSize do
			local group = select(3, GetRaidRosterInfo(i))
			maxGroup = math_max(maxGroup, group)
		end
		if (maxGroup > 5) or (raidSize > 25) then
			layoutSize = 40
		else
			if 5 * maxGroup == 20 then
				layoutSize = 25
			else
				layoutSize =  5 * maxGroup
			end
		end

	elseif GetNumPartyMembers() > 0 then
		layoutSize = 5
	end
	
	if layoutSize == 0 then
		newLayout = "None"
	else
		newLayout = "By Group "..layoutSize
		newLayout = getWithPetsLayout(newLayout)
	end
	
	newLayout = getLocalizedLayout(newLayout)
	
	local lastLayout = self.db.profile.lastLayout
	
	if not newLayout or newLayout == lastLayout then
		return
	end
	
	-- if zoneLock is up, don't change layout size
	-- if we can find layoutSize in lastLayout, it means we're just changing to/from pet layout
	if zoneLock and not string_find(lastLayout, layoutSize) then
		return
	end

	self.db.profile.lastLayout = newLayout
	GridLayout:LoadLayout(newLayout)
end
